//-----------------------------------------------------------------------------
// Group method definitions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/Group.h"


namespace Cheetah {

//-----------------------------------------------------------------------------
//
// Group Constructors
//
// Initialize the nContexts and myContext values and copy the ranks array
// if provided.  If nContexts == -1 and myContext == NOT_A_MEMBER, the group
// is in an invalid state and must be later initialized to valid values.
// Also note that ranks_m must be initialized (e.g., to NULL) before calling
// initialize().
//
//-----------------------------------------------------------------------------

Group::Group(int nContexts, int myContext, int* ranks, int id)
  : ranks_m(0)
{
  initialize(nContexts, myContext, ranks, id);
}


//-----------------------------------------------------------------------------
//
// Group Destructor.
//
// Frees memory for the ranks array.
//
//-----------------------------------------------------------------------------

Group::~Group()
{
  if (ranks_m != 0) delete[] ranks_m;
}


//-----------------------------------------------------------------------------
//
// Initializes the group to the given values.  Deletes the old ranks array
// if it exists and copies the incoming one (or creates the default ordering).
//
//-----------------------------------------------------------------------------

void Group::initialize(int nContexts, int myContext, int* ranks, int id)
{
  CheetahRTAssert(nContexts > 0
		  && (myContext_m == NOT_A_MEMBER || myContext_m >= 0));

  nContexts_m = nContexts;
  myContext_m = myContext;
  id_m = id;

  if (ranks_m != 0) delete[] ranks_m;

  ranks_m = new int[nContexts_m];

  if (ranks == 0) {
    for (int i = 0; i < nContexts_m; i++) ranks_m[i] = i;
  } else {
    for (int i = 0; i < nContexts_m; i++) ranks_m[i] = ranks[i];
  }
}


} // namespace Cheetah
