//-----------------------------------------------------------------------------
// MPICommData method definitions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/MPI/MPICommData.h"

namespace Cheetah {


//-----------------------------------------------------------------------------
//
// MPICommData Constructor
//
//-----------------------------------------------------------------------------

MPICommData::MPICommData(int nContexts)
{
  next_put_tag_m = 0;
  next_get_tag_m = 0;

  //
  // Initialize the arrays that determine the next slot to use in the
  // barrier for each remote CPU.
  //
  barrier_counters_m[0] = 0;
  barrier_counters_m[1] = 0;

  next_barrier_m = new int[nContexts];
  for (int i = 0; i < nContexts; ++i) {
    next_barrier_m[i] = 0;
  }
}


MPICommData::~MPICommData()
{
  delete[] next_barrier_m;
}


//----------------------------------------------------------------------
//
// next_put_tag 
//
// Generate the next unique tag for a put.  If we wrap around the tag
// space, start at the beginning but be sure to skip the special
// PUT_REQUEST_TAG!
//
//----------------------------------------------------------------------

int MPICommData::next_put_tag()
{
  next_put_tag_m += 1;
  if ( next_put_tag_m == MPI_TAG_UB ) {
    next_put_tag_m = PUT_REQUEST_TAG + 1;
  }
  return next_put_tag_m;
}


//----------------------------------------------------------------------
//
// next_get_tag 
//
// Generate the next unique tag for a get.  If we wrap around the tag
// space, start at the beginning but be sure to skip the special
// GET_REQUEST_TAG!
//
//----------------------------------------------------------------------

int MPICommData::next_get_tag()
{
  next_get_tag_m += 1;
  if ( next_get_tag_m == MPI_TAG_UB ) {
    next_get_tag_m = GET_REQUEST_TAG + 1;
  }
  return next_get_tag_m;
}

} // namespace Cheetah
