#ifndef CHEETAH_CONTROLLER_MPI_MPICONTROLLER_H
#define CHEETAH_CONTROLLER_MPI_MPICONTROLLER_H

//-----------------------------------------------------------------------------
// Classes:
//   MPIController
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Overview:
//
// A ControllerImpl which uses MPI for distributed-memory communication.
// This class primarily delegates operations to the cheetah_mpi functions.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/Controller.h"
#include "Controller/MPI/MPIGroup.h"


namespace Cheetah {

//-----------------------------------------------------------------------------
//
// class MPIController
//
//-----------------------------------------------------------------------------

class MPIController : public ControllerImpl
{
public:

  //
  // Constructor
  //
  // We input the command line arguments and parse them to find the
  // degree of parallelism.  Sometimes the calling application must
  // call MPI_Init directly, in this case set init_MPI to false.
  //
  MPIController(int &argc, char **&argv, bool init_MPI);

  //
  // Destructor
  //
  // Tell the underlying library to tear down the parallelism.
  //
  virtual ~MPIController();

  //
  // We are filling in the whole interface from ControllerImpl.
  // Each of these functions primarily delegates its operation to the
  // cheetah_mpi library.
  //

  virtual void ainvoke(int context, int tag, void *buffer, int len,
		       int *local_bell);

  virtual void put(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void get(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void poll();

  virtual void wait(volatile int *bell, int value);

  virtual void barrier();

  virtual void barrier(const Group& group);

  virtual
  const Group& createGroup(const Group& oldGroup, int nContexts, int* ranks);

  //
  // Returns a pointer to the MPIGroup associated with the given id
  //

  inline const MPIGroup* mpi_group(int id = 0) const
  {
    const MPIGroup* g = dynamic_cast<const MPIGroup*>( groups_m[id] );
    CheetahRTAssert(g);
    return g;
  }

  //
  // The subclasses will register themselves by giving us a factory
  // method to call to create that subclass together with a string to
  // find on the command line.
  //

  static void registerFactory();

 private:

  bool initMPI_m;

};


} // namespace Cheetah

#endif // CHEETAH_CONTROLLER_MPI_MPICONTROLLER_H
