//-----------------------------------------------------------------------------
// Group method definitions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/MPI/MPIGroup.h"

namespace Cheetah {

//-----------------------------------------------------------------------------
//
// MPIGroup Constructors
//
// Initialize the nContexts and myContext values and copy the ranks array
// if provided.  If nContexts == -1 and myContext == NOT_A_MEMBER, the group
// is in an invalid state and must be later initialized to valid values.
// Also note that ranks_m must be initialized (e.g., to NULL) before calling
// initialize().
//
//-----------------------------------------------------------------------------

MPIGroup::MPIGroup(MPI_Comm comm, int id)
  : Group()
{
  int nContexts, myContext;

  MPI_Comm_size(comm, &nContexts);
  MPI_Comm_rank(comm, &myContext);

  Group::initialize(nContexts, myContext, 0, id);

  mpi_comm_m = comm;
  MPI_Comm_dup(comm, &rfi_comm_m);
  MPI_Comm_dup(comm, &get_comm_m);
  MPI_Comm_dup(comm, &put_comm_m);
}


MPIGroup::MPIGroup(const Group& oldGroup, int nContexts, int* ranks, int id)
  : Group()
{
  MPI_Group mpiOldGroup, mpiNewGroup;

  const MPIGroup* old = dynamic_cast<const MPIGroup*>(&oldGroup);
  CheetahRTAssert(old);

  MPI_Comm oldComm = old->MPICommunicator();

  MPI_Comm_group(oldComm, &mpiOldGroup);
  MPI_Group_incl(mpiOldGroup, nContexts, ranks, &mpiNewGroup);
  MPI_Comm_create(oldComm, mpiNewGroup, &mpi_comm_m);

  MPI_Comm_create(oldComm, mpiNewGroup, &rfi_comm_m);
  MPI_Comm_create(oldComm, mpiNewGroup, &get_comm_m);
  MPI_Comm_create(oldComm, mpiNewGroup, &put_comm_m);

  //
  // check to make sure the caller is member of the group
  //

  int rank = oldGroup.mycontext();
  int myContext = NOT_A_MEMBER;
  for (int i = 0; i < nContexts; i++) {
    if (rank == ranks[i]) {
      MPI_Comm_rank(mpi_comm_m, &myContext);
      break;
    }
  }
  Group::initialize(nContexts, myContext, ranks, id);
}

MPIGroup::~MPIGroup()
{
}

} // namespace Cheetah
