#ifndef CHEETAH_CONTROLLER_MPI_MPIGROUP_H
#define CHEETAH_CONTROLLER_MPI_MPIGROUP_H

//-----------------------------------------------------------------------------
// Classes:
//   MPIGroup
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Overview:
//
// A subclass of the group.  This class must maintain MPI specific information
// like the MPI Communicator associated with the group.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/Group.h"
#include "Controller/MPI/cheetah_mpi.h"
#include <mpi.h>


namespace Cheetah {

//-----------------------------------------------------------------------------
//
// class MPIGroup
//
//-----------------------------------------------------------------------------

class MPIGroup : public Group
{
public:

  //
  // Constructors.  As provided, the default constructor creates an invalid
  // state.  The MPIGroup cannot be corrected with the initialize function.
  // So declare but do provide a definition.
  //

  MPIGroup();
  
  MPIGroup(MPI_Comm comm, int id);

  //
  // Construct an MPIGroup from a Group which must be downcastable to
  // an MPIGroup.
  //

  MPIGroup(const Group& oldGroup, int nContexts, int* ranks, int id);

  //
  // Declare but don't define to insure that it can't be used
  //
  MPIGroup(const Group& g);

  //
  // Destructor.
  //
  virtual ~MPIGroup();

  //
  // Declare but don't define to insure that it can't be used
  //

  MPIGroup& operator=(const MPIGroup& group);

  //
  // Accessor methods
  //

  virtual bool isMPIGroup() const { return true; }

  virtual MPI_Comm MPICommunicator() const { return mpi_comm_m; }
  inline  MPI_Comm RFICommunicator() const { return rfi_comm_m; }
  inline  MPI_Comm getCommunicator() const { return get_comm_m; }
  inline  MPI_Comm putCommunicator() const { return put_comm_m; }

protected:

  //
  // Four communicators for keeping the messages straight.
  // mpi_comm_m: base communicator others derived from, given to outside users
  // rfi_comm_m: all messages about remote function invocation.
  // get_comm_m: all get messages, including requests and replies.
  // put_comm_m: all put messages.
  //

  MPI_Comm mpi_comm_m;		// communicator for group
  MPI_Comm rfi_comm_m;		// communicator for group rfi messages
  MPI_Comm get_comm_m;		// communicator for group get messages
  MPI_Comm put_comm_m;		// communicator for group put messages

};
} // namespace Cheetah

#endif // CHEETAH_CONTROLLER_MPI_MPIGROUP_H
