#ifndef CHEETAH_CONTROLLER_SERIALCONTROLLER_H
#define CHEETAH_CONTROLLER_SERIALCONTROLLER_H

//-----------------------------------------------------------------------------
// Classes:
//   SerialController
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Overview:
//
// A ControllerImpl which just looks like one context.  This is
// a default case for codes that want to use a controller in their
// code but don't want to actually use a particular communication library
// during the run.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/Controller.h"


namespace Cheetah {

//-----------------------------------------------------------------------------
//
// class SerialController
//
//-----------------------------------------------------------------------------

class SerialController : public ControllerImpl
{
public:

  //
  // Constructor
  //
  // We input the command line arguments and parse them to find the
  // degree of parallelism.
  //
  SerialController(int &argc, char **&argv);

  //
  // Destructor
  //
  // Tell the underlying library to tear down the parallelism.
  //
  ~SerialController();

  //
  // We are filling in the whole interface from ControllerImpl.
  // For serial, this does very little.
  //

  virtual void ainvoke(int context, int tag, void *buffer, int len,
		       int *local_bell);

  virtual void put(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void get(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void poll();

  virtual void wait(volatile int *bell, int value);

  virtual void barrier();

  virtual void barrier(const Group& group);

  //
  // The subclasses will register themselves by giving us a factory
  // method to call to create that subclass together with a string to
  // find on the command line.
  //

  static void registerFactory();
};


} // namespace Cheetah

#endif // CHEETAH_CONTROLLER_SERIALCONTROLLER_H

// ACL:rcsinfo
// ----------------------------------------------------------------------
// $RCSfile: SerialController.h,v $   $Author: rasmussn $
// $Revision: 1.2 $   $Date: 2000/08/16 23:04:26 $
// ----------------------------------------------------------------------
// ACL:rcsinfo
