#ifndef CHEETAH_CONTROLLER_SHMEM_MMALLOCATOR_H
#define CHEETAH_CONTROLLER_SHMEM_MMALLOCATOR_H

//-----------------------------------------------------------------------------
// Classes:
//   MM_Allocator
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include <mm.h>


//-----------------------------------------------------------------------------
//
// class MM_Allocator
//
//-----------------------------------------------------------------------------

template<class T>
class MM_Allocator
{
public:

  //
  // Allocator typedefs.
  //
  typedef T value_type;
  typedef T* pointer;
  typedef const T* const_pointer;
  typedef T& reference;
  typedef const T& const_reference;
  typedef int difference_type;
  typedef int size_type;
  template<class U> struct rebind { typedef MM_Allocator<U> other; };

  //
  // Constructors and destructors don't need to do anything because we
  // don't store any data.
  //
  MM_Allocator() {}
  MM_Allocator(const MM_Allocator<T>&) {}
  template<class U>  MM_Allocator(const MM_Allocator<U>&) {}
  ~MM_Allocator() {}

  //
  // We can allocate and delete freely between MM_Allocators.
  //
  template<class U> bool operator==(MM_Allocator<U>) { return true; }

  //
  // Allocate from shared memory.
  //
  pointer allocate(int n) { return (pointer)MM_malloc(n*sizeof(T)); }

  template<class P>
  void allocate(int n, P) { return (pointer)MM_malloc(n*sizeof(T)); }

  //
  // Free back to shared memory.
  //
  void deallocate(pointer x, int) { MM_free(x); }

  //
  // Return the maximum size that can be allocated.
  //
  size_type max_size() { return MM_maxsize(); }
  
  //
  // Construct an object of type T at a given location.
  //
  void construct(pointer p, const T& t) { new (p) T(t); }

  //
  // Destroy an object of type T at a given location.
  //
  void destroy(pointer p) { p->~T(); }

  //
  // Take the address of an object.
  //
  pointer address(T& t) { return &t; }
  const_pointer address(const T& t) { return &t; }
};

#endif

// ACL:rcsinfo
// ----------------------------------------------------------------------
// $RCSfile: MM_Allocator.h,v $   $Author: rasmussn $
// $Revision: 1.1 $   $Date: 2000/04/17 19:33:09 $
// ----------------------------------------------------------------------
// ACL:rcsinfo
