//-----------------------------------------------------------------------------
// ShmemController method definitions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/Shmem/ShmemController.h"
#include "Controller/Shmem/cheetah_shmem.h"
#include "Controller/ControllerFactory.h"
#include "Controller/Group.h"

namespace Cheetah {

//-----------------------------------------------------------------------------
//
// ShmemController::Constructor
//
// Pass the command line arguments to the cheetah_shmem library to set
// up the parallelism.
//
//-----------------------------------------------------------------------------

ShmemController::ShmemController(int &argc, char **&argv)
{
  cheetah_shmem_setup(&argc, &argv);

  // create the initial group (group world)
  groups_m.push_back(
          new Group(cheetah_shmem_ncontexts(), cheetah_shmem_mycontext())
                    );
}


//-----------------------------------------------------------------------------
//
// ShmemController::Destructor
//
// Tell the underlying library to tear down the parallelism.
//
//-----------------------------------------------------------------------------

ShmemController::~ShmemController()
{
  cheetah_shmem_finalize();
}


//-----------------------------------------------------------------------------
//
// ShmemController::ainvoke
//
// To call a remote function, just tell the underlying library to do
// it.
//
//-----------------------------------------------------------------------------

void
ShmemController::ainvoke(int context, int tag, void *buf, int len, int *bell)
{
  cheetah_shmem_ainvoke(context,tag,buf,len,bell);
}


//-----------------------------------------------------------------------------
//
// ShmemController::put
//
// To put memory, just tell the underlying library to do it.
//
//-----------------------------------------------------------------------------

void
ShmemController::put(int c, void *r, void *l, int len, int *lb, int *rb)
{
  cheetah_shmem_put(c,r,l,len,lb,rb);
}


//-----------------------------------------------------------------------------
//
// ShmemController::get
//
// To get memory, just tell the underlying library to do it.
//
//-----------------------------------------------------------------------------

void
ShmemController::get(int c, void *r, void *l, int len, int *lb, int *rb)
{
  cheetah_shmem_get(c,r,l,len,lb,rb);
}


//-----------------------------------------------------------------------------
//
// ShmemController::poll
//
// To handle incoming messages, just tell the underlying library to do
// it.
//
//-----------------------------------------------------------------------------

void
ShmemController::poll()
{
  cheetah_shmem_poll();
}


//-----------------------------------------------------------------------------
//
// ShmemController::wait
//
// To wait for a bell to reach a given value, just tell the underlying
// library to do it.
//
//-----------------------------------------------------------------------------

void
ShmemController::wait(volatile int *bell, int value)
{
  cheetah_shmem_wait(bell,value);
}


//-----------------------------------------------------------------------------
//
// ShmemController::barrier
//
// To do a barrier, just tell the underlying library to do it.
//
//-----------------------------------------------------------------------------

void
ShmemController::barrier()
{
  cheetah_shmem_barrier();
}


void
ShmemController::barrier(const Group& group)
{
  cheetah_shmem_barrier();
}


//-----------------------------------------------------------------------------
//
// The function used to create new implementations.  This is registered
// with a factory.
//
//-----------------------------------------------------------------------------

static ControllerImpl*
shmemControllerFactory(int &argc, char ** &argv)
{
  return new ShmemController(argc,argv);
}


//-----------------------------------------------------------------------------
//
// ShmemController::registerFactory
//
// Register this subclass with the controller factory list.
//
//-----------------------------------------------------------------------------

void
ShmemController::registerFactory()
{
  ControllerFactory::registerFactory("-shmem", shmemControllerFactory);
}


} // namespace Cheetah
