#ifndef CHEETAH_CONTROLLER_SHMEM_SHMEMCONTROLLER_H
#define CHEETAH_CONTROLLER_SHMEM_SHMEMCONTROLLER_H

//-----------------------------------------------------------------------------
// Classes:
//   ShmemController
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Overview:
//
// A ControllerImpl which uses fork to produce parallelism and
// shared memory for communication.  This class mostly just delegates
// is operations to the cheetah_shmem functions.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/Controller.h"


namespace Cheetah {

//-----------------------------------------------------------------------------
//
// class ShmemController
//
//-----------------------------------------------------------------------------

class ShmemController : public ControllerImpl
{
public:

  //
  // Constructor
  //
  // We input the command line arguments and parse them to find the
  // degree of parallelism.
  //
  ShmemController(int &argc, char **&argv);

  //
  // Destructor
  //
  // Tell the underlying library to tear down the parallelism.
  //
  ~ShmemController();

  //
  // We are filling in the whole interface from ControllerImpl.
  // Each of these functions just delegates its operation to the
  // cheetah_shmem library.
  //

  virtual void ainvoke(int context, int tag, void *buffer, int len,
		       int *local_bell);

  virtual void put(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void get(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void poll();

  virtual void wait(volatile int *bell, int value);

  virtual void barrier();

  virtual void barrier(const Group& group);

  //
  // The subclasses will register themselves by giving us a factory
  // method to call to create that subclass together with a string to
  // find on the command line.
  //

  static void registerFactory();
};


} // namespace Cheetah

#endif // CHEETAH_CONTROLLER_SHMEM_SHMEMCONTROLLER_H
