#ifndef CHEETAH_CONTROLLER_SHMEM_SHMEM_H
#define CHEETAH_CONTROLLER_SHMEM_SHMEM_H

/*
//-----------------------------------------------------------------------------
// Overview:
//
// C interface routines for implementing Cheetah mechanism on Shmem, using MM.
//
//-----------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C"
{
#endif

  /*
  // Set up and tear down the messaging system.  This initializes and
  // shuts down the shared memory region and the forked processes.
  // 
  */
  void cheetah_shmem_setup(int *, char ***);
  void cheetah_shmem_finalize();

  /*
  // Report on the number of contexts and which one this is.
  */
  int cheetah_shmem_ncontexts();
  int cheetah_shmem_mycontext();

  /*
  // Call a remote function with the given tag and buffer of data.  Do
  // not wait for a reply, but ring a bell when the buffer may be
  // reused.
  */
  void cheetah_shmem_ainvoke(int context, int tag, void *buffer, int len,
			   int *local_bell);

  /*
  // Copy a local buffer to a remote one in the specified context.
  // When the message arrives, ring a bell over there by incrementing
  // it.  When the buffer may be reused, ring a bell locally.
  */
  void cheetah_shmem_put(int context, void *remote, void *local, int len,
		       int *local_bell, int *remote_bell);

  /*
  // Copy a remote buffer to a local one.  When the data arrives, ring
  // a local bell.  When it has been successfully sent from the far
  // side, ring a bell over there.
  */
  void cheetah_shmem_get(int context, void *remote, void *local, int len,
		       int *local_bell, int *remote_bell);

  /*
  // Have the system pull things out of the network and handle them.
  // Keeps going as long as there are things to do.
  */


  void cheetah_shmem_poll();

  /*
  // Wait for a doorbell to ring (get to a given value) while
  // processing messages.
  */
  void cheetah_shmem_wait(volatile int *bell, int value);

  /*
  // Do a barrier among all of the contexts.
  */
  void cheetah_shmem_barrier();

#ifdef __cplusplus
}
#endif

#endif

/*
// ACL:rcsinfo
// ----------------------------------------------------------------------
// $RCSfile: cheetah_shmem.h,v $   $Author: rasmussn $
// $Revision: 1.2 $   $Date: 2000/05/02 22:48:19 $
// ----------------------------------------------------------------------
// ACL:rcsinfo
*/
