//-----------------------------------------------------------------------------
// ULMController method definitions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/ULM/ULMController.h"
#include "Controller/ULM/cheetah_ulm.h"
#include "Controller/ControllerFactory.h"
#include "Controller/Group.h"


namespace Cheetah {

//-----------------------------------------------------------------------------
//
// ULMController::Constructor
//
// Pass the command line arguments to the cheetah_ulm library to set
// up the parallelism.
//
//-----------------------------------------------------------------------------

ULMController::ULMController(int &argc, char **&argv)
{
  cheetah_ulm_setup(&argc,&argv);

  Group* group = new Group(cheetah_ulm_ncontexts(), cheetah_ulm_mycontext());
  groups_m.push_back(group);
}


//-----------------------------------------------------------------------------
//
// ULMController::Destructor
//
// Tell the underlying library to tear down the parallelism.
//
//-----------------------------------------------------------------------------

ULMController::~ULMController()
{
  cheetah_ulm_finalize();
}


//-----------------------------------------------------------------------------
//
// ULMController::ainvoke
//
// To call a remote function, just tell the underlying library to do
// it.
//
//-----------------------------------------------------------------------------

void
ULMController::ainvoke(int context, int tag, void *buf, int len, int *bell)
{
  cheetah_ulm_ainvoke(context,tag,buf,len,bell);
}


//-----------------------------------------------------------------------------
//
// ULMController::put
//
// To put memory, just tell the underlying library to do it.
//
//-----------------------------------------------------------------------------

void
ULMController::put(int c, void *r, void *l, int len, int *lb, int *rb)
{
  cheetah_ulm_put(c,r,l,len,lb,rb);
}


//-----------------------------------------------------------------------------
//
// ULMController::get
//
// To get memory, just tell the underlying library to do it.
//
//-----------------------------------------------------------------------------

void
ULMController::get(int c, void *r, void *l, int len, int *lb, int *rb)
{
  cheetah_ulm_get(c,r,l,len,lb,rb);
}


//-----------------------------------------------------------------------------
//
// ULMController::poll
//
// To handle incoming messages, just tell the underlying library to do
// it.
//
//-----------------------------------------------------------------------------

void
ULMController::poll()
{
  cheetah_ulm_poll();
}


//-----------------------------------------------------------------------------
//
// ULMController::wait
//
// To wait for a bell to reach a given value, just tell the underlying
// library to do it.
//
//-----------------------------------------------------------------------------

void
ULMController::wait(volatile int *bell, int value)
{
  cheetah_ulm_wait(bell,value);
}


//-----------------------------------------------------------------------------
//
// ULMController::barrier
//
// To do a barrier, just tell the underlying library to do it.
//
//-----------------------------------------------------------------------------

void
ULMController::barrier()
{
  cheetah_ulm_barrier();
}


//-----------------------------------------------------------------------------
//
// The function used to create new implementations.  This is registered
// with a factory.
//
//-----------------------------------------------------------------------------

static ControllerImpl*
ulmControllerFactory(int &argc, char ** &argv)
{
  return new ULMController(argc,argv);
}


//-----------------------------------------------------------------------------
//
// ULMController::registerFactory
//
// Register this subclass with the controller factory list.
//
//-----------------------------------------------------------------------------

void
ULMController::registerFactory()
{
  ControllerFactory::registerFactory("-ulm", ulmControllerFactory);
}


} // namespace Cheetah
