#ifndef CHEETAH_CONTROLLER_ULM_ULMCONTROLLER_H
#define CHEETAH_CONTROLLER_ULM_ULMCONTROLLER_H

//-----------------------------------------------------------------------------
// Classes:
//   ULMController
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Overview:
//
// A ControllerImpl which uses the ULM message-passing library.
// This class mostly just delegates is operations to the cheetah_ulm functions.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

#include "Controller/Controller.h"


namespace Cheetah {

//-----------------------------------------------------------------------------
//
// class ULMController
//
//-----------------------------------------------------------------------------

class ULMController : public ControllerImpl
{
public:

  //
  // Constructor
  //
  // We input the command line arguments and parse them to find the
  // degree of parallelism.
  //
  ULMController(int &argc, char **&argv);

  //
  // Destructor
  //
  // Tell the underlying library to tear down the parallelism.
  //
  ~ULMController();

  //
  // We are filling in the whole interface from ControllerImpl.
  // Each of these functions just delegates its operation to the
  // cheetah_ulm library.
  //

  virtual void ainvoke(int context, int tag, void *buffer, int len,
		       int *local_bell);

  virtual void put(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void get(int context, void *remote, void *local, int len,
		   int *local_bell, int *remote_bell);

  virtual void poll();

  virtual void wait(volatile int *bell, int value);

  virtual void barrier();

  //
  // The subclasses will register themselves by giving us a factory
  // method to call to create that subclass together with a string to
  // find on the command line.
  //

  static void registerFactory();
};


} // namespace Cheetah

#endif // CHEETAH_CONTROLLER_ULM_ULMCONTROLLER_H
