#ifndef CHEETAH_CONTROLLER_ULM_ULM_H
#define CHEETAH_CONTROLLER_ULM_ULM_H

/*
//-----------------------------------------------------------------------------
// Overview:
//
// C interface routines for implementing Cheetah mechanism on ULM.
//
//-----------------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C"
{
#endif

  /*
  // Set up and tear down the messaging system.  This initializes and
  // shuts down ULM.
  // 
  */
  void cheetah_ulm_setup(int *, char ***);
  void cheetah_ulm_finalize();

  /*
  // Report on the number of contexts and which one this is.
  */
  int cheetah_ulm_ncontexts();
  int cheetah_ulm_mycontext();

  /*
  // Call a remote function with the given tag and buffer of data.  Do
  // not wait for a reply, but ring a bell when the buffer may be
  // reused.
  */
  void cheetah_ulm_ainvoke(int context, int tag, void *buffer, int len,
			 int *local_bell);

  /*
  // Copy a local buffer to a remote one in the specified context.
  // When the message arrives, ring a bell over there by incrementing
  // it.  When the buffer may be reused, ring a bell locally.
  */
  void cheetah_ulm_put(int context, void *remote, void *local, int len,
		     int *local_bell, int *remote_bell);

  /*
  // Copy a remote buffer to a local one.  When the data arrives, ring
  // a local bell.  When it has been successfully sent from the far
  // side, ring a bell over there.
  */
  void cheetah_ulm_get(int context, void *remote, void *local, int len,
		     int *local_bell, int *remote_bell);

  /*
  // Have the system pull things out of the network and handle them.
  // Keeps going as long as there are things to do.
  */
  void cheetah_ulm_poll();

  /*
  // Wait for a doorbell to ring (get to a given value) while
  // processing messages.
  */
  void cheetah_ulm_wait(volatile int *bell, int value);

  /*
  // Do a barrier among all of the contexts.
  */
  void cheetah_ulm_barrier();

#ifdef __cplusplus
}
#endif

#endif
