#ifndef CHEETAH_CORE_AINVOKE_H
#define CHEETAH_CORE_AINVOKE_H

//-----------------------------------------------------------------------------
// C++ function interface to Cheetah capabilities.
//
// NOTE: The following routines are based on an older design that used
// a 'core' interface and these C++ functions as wrappers around that
// core.  This design has since been changed to use polymorphic 'controllers'
// for the core functionality, and these routines have NOT been updated to
// that new design.  They are here for historical reference.  Because
// of this, they are disabled with an #if 0 / #endif block.
//
//-----------------------------------------------------------------------------

#if 0

//-----------------------------------------------------------------------------
// Include Files
//-----------------------------------------------------------------------------

#include <assert.h>
#include <cheetah_core.h>


//----------------------------------------------------------------------

inline void
cheetah_ainvoke(int context, int tag, int *bell)
{
  assert((context>=0)&&(context<cheetah_core_ncontexts()));
  cheetah_core_ainvoke(context,tag,0,0,0);
  if ( bell )
    ++*bell;
}

//----------------------------------------------------------------------

template<class T1>
void cheetah_ainvoke(int context, int tag, int *bell,
		   const T1& x1)
{
  assert((context>=0)&&(context<cheetah_core_ncontexts()));
  int length = cheetah_packed_size(x1);
  assert(length>=0);
  char *buffer = new char[length];
  int n = cheetah_pack(x1,buffer,length);
  assert( n<=length );
  int my_bell = 0;
  cheetah_core_ainvoke(context,tag,buffer,n,&my_bell);
  cheetah_core_wait(&my_bell,1);
  delete [] buffer;
  if ( bell )
    ++*bell;
}

//----------------------------------------------------------------------

template<class T1, class T2>
void cheetah_ainvoke(int context, int tag, int *bell,
		   const T1 &x1, const T2 &x2)
{
  assert((context>=0)&&(context<cheetah_core_ncontexts()));
  int length = cheetah_packed_size(x1) + cheetah_packed_size(x2);
  assert(length>=0);
  char *buffer = new char[length];
  int n = cheetah_pack(x1,buffer,length);
  assert( n<=length );
  n += cheetah_pack(x2,buffer+n,length-n);
  assert( n<=length );
  cheetah_core_ainvoke(context,tag,buffer,n,0);
  delete [] buffer;
  if ( bell )
    ++*bell;
}

//----------------------------------------------------------------------

template<class T1, class T2, class T3>
void cheetah_ainvoke(int context, int tag, int *bell,
		   const T1 &x1, const T2 &x2, const T3 &x3)
{
  assert((context>=0)&&(context<cheetah_core_ncontexts()));
  int length =
    cheetah_packed_size(x1) + cheetah_packed_size(x2) + cheetah_packed_size(x3);
  assert(length>=0);
  char *buffer = new char[length];
  int n = cheetah_pack(x1,buffer,length);
  assert( n<=length );
  n += cheetah_pack(x2,buffer+n,length-n);
  assert( n<=length );
  n += cheetah_pack(x3,buffer+n,length-n);
  assert( n<=length );
  int my_bell = 0;
  cheetah_core_ainvoke(context,tag,buffer,n,&my_bell);
  cheetah_core_wait(&my_bell,1);
  delete [] buffer;
  if ( bell )
    ++*bell;
}

//----------------------------------------------------------------------

template<class T1, class T2, class T3, class T4>
void cheetah_ainvoke(int context, int tag, int *bell,
		   const T1 &x1, const T2 &x2, const T3 &x3, const T4 &x4)
{
  assert((context>=0)&&(context<cheetah_core_ncontexts()));
  int length =
    cheetah_packed_size(x1) + cheetah_packed_size(x2) +
    cheetah_packed_size(x3) + cheetah_packed_size(x4);
  assert(length>=0);
  char *buffer = new char[length];
  int n = cheetah_pack(x1,buffer,length);
  assert( n<=length );
  n += cheetah_pack(x2,buffer+n,length-n);
  assert( n<=length );
  n += cheetah_pack(x3,buffer+n,length-n);
  assert( n<=length );
  n += cheetah_pack(x4,buffer+n,length-n);
  assert( n<=length );
  int my_bell = 0;
  cheetah_core_ainvoke(context,tag,buffer,n,&my_bell);
  cheetah_core_wait(&my_bell,1);
  delete [] buffer;
  if ( bell )
    ++*bell;
}

//----------------------------------------------------------------------

#endif   // #if 0

#endif // CHEETAH_CORE_AINVOKE_H
