#ifndef CHEETAH_CORE_OPERATE_H
#define CHEETAH_CORE_OPERATE_H

//-----------------------------------------------------------------------------
// C++ function interface to Cheetah capabilities.
//
// NOTE: The following routines are based on an older design that used
// a 'core' interface and these C++ functions as wrappers around that
// core.  This design has since been changed to use polymorphic 'controllers'
// for the core functionality, and these routines have NOT been updated to
// that new design.  They are here for historical reference.  Because
// of this, they are disabled with an #if 0 / #endif block.
//
//-----------------------------------------------------------------------------

#if 0

//-----------------------------------------------------------------------------
// Include Files
//-----------------------------------------------------------------------------

#include <assert.h>
#include <cheetah_core.h>
#include <cheetah_register.h>
#include <cheetah_ainvoke.h>


//----------------------------------------------------------------------

template<class T>
struct CheetahOperate
{
  static void operate(int context, T* remote, int op);
  static void handle_request(T* destination, int op);
  static const int request_tag_s;
  enum { INCREMENT=1, DECREMENT };
};

template<class T>
const int CheetahOperate<T>::request_tag_s = 
  cheetah_register(CheetahOperate<T>::handle_request,
		 "handle_operate_request");


//----------------------------------------------------------------------
//
// CheetahOperate::operate
//
// Take a reference to a local object and operate it to a remote context
// at a given location.  Us the full object pack/unpack techniques.
//
//----------------------------------------------------------------------

template<class T>
void
CheetahOperate<T>::operate(int context, T* remote, int op) 
{
  assert( (context>=0) && (context<cheetah_core_ncontexts()) );
  assert( (op==INCREMENT) || (op==DECREMENT) );
  assert( remote!=0 );

  //
  // A bell to ring when the object is on its way.
  //
  int bell=0;

  //
  // Send the object to the other context.
  //
  cheetah_ainvoke(context, request_tag_s, &bell, remote, op);

  //
  // When it is on its way it will ring the bell.
  //
  cheetah_core_wait(&bell,1);
}

//----------------------------------------------------------------------

template<class T>
void CheetahOperate<T>::
handle_request(T* destination, int op)
{
  assert(destination!=0);
  switch (op)
    {
    case INCREMENT:
      ++*destination;
      break;
    case DECREMENT:
      --*destination;
      break;
    default:
      fprintf(stderr,"%d: Invalid operate request %d!\n",
	      cheetah_core_mycontext(),op);
      break;
    }
}

//----------------------------------------------------------------------

#endif   // #if 0

#endif // CHEETAH_CORE_OPERATE_H
