#ifndef CHEETAH_CORE_PACK_H
#define CHEETAH_CORE_PACK_H

//----------------------------------------------------------------------
//
// cheetah_packed_size
// cheetah_pack
// cheetah_unpack
// cheetah_delete_unpacked
//
// Functions for packing and unpacking user classes from communication
// buffers.  These are the defaults, and the copy ctor is enough for
// pack, and that it is OK to just refer to the object in the buffer
// for unpack.  Classes for which that is not the case should define
// their own.
//
// NOTE: The following routines are based on an older design that used
// a 'core' interface and these C++ functions as wrappers around that
// core.  This design has since been changed to use polymorphic 'controllers'
// for the core functionality, and these routines have NOT been updated to
// that new design.  They are here for historical reference.  Because
// of this, they are disabled with an #if 0 / #endif block.
//
//----------------------------------------------------------------------

#if 0

//-----------------------------------------------------------------------------
// Include Files
//-----------------------------------------------------------------------------

#include <string.h>
#include <new>
#include <typeinfo>


#define CHEETAH_PACK_ALIGNMENT (sizeof(void*))


//-----------------------------------------------------------------------------
//
// cheetah_packed_size computes the size of an item of type T, including any extra
// space needed for memory alignment.
//
//-----------------------------------------------------------------------------

template<class T>
inline int cheetah_packed_size(const T&)
{
  const int off = sizeof(T)%CHEETAH_PACK_ALIGNMENT;
  int s= sizeof(T) + ( off==0 ? 0 : CHEETAH_PACK_ALIGNMENT - off ) ;
  return s;
}


//-----------------------------------------------------------------------------
//
// cheetah_pack takes an item of type T and copies it into the given buffer.  It
// returns the number of bytes used in packing, including any space that was
// needed for memory alignment.  The third argument should contain the current
// length of the buffer, and can be used for error-checking.
//
//-----------------------------------------------------------------------------

template<class T>
inline int cheetah_pack(const T& p, char* buffer, int length)
{
  new ((void*)buffer) T(p);
  return cheetah_packed_size(p);
}


//-----------------------------------------------------------------------------
//
// cheetah_unpack takes a pointer to an item of type T, unpacks an item from the
// given buffer, and sets the provided pointer equal to the start of that
// unpacked element.  This particular version just takes the current buffer
// pointer and casts it to the proper type, assuming the data in the buffer
// is the binary image of an item of that type.  The third arg should contain
// the current length of the buffer, and can be used for error-checking.
//
//-----------------------------------------------------------------------------

template<class T>
inline int cheetah_unpack(T* &p, char* buffer, int length)
{
  p = (T*)buffer;
  return cheetah_packed_size(*p);
}


//-----------------------------------------------------------------------------
//
// cheetah_delete_unpacked cleans up an unpacked item, by running its destructor.
//
//-----------------------------------------------------------------------------

template<class T>
inline void cheetah_delete_unpacked(T* p)
{
  p->~T();
}

#endif   // #if 0

#endif // CHEETAH_CORE_PACK_H
