//----------------------------------------------------------------------
// Definitions for the static members of CheetahFunctionEntry.
//
// NOTE: The following routines are based on an older design that used
// a 'core' interface and these C++ functions as wrappers around that
// core.  This design has since been changed to use polymorphic 'controllers'
// for the core functionality, and these routines have NOT been updated to
// that new design.  They are here for historical reference.  Because
// of this, they are disabled with an #if 0 / #endif block.
//
//-----------------------------------------------------------------------------

#if 0

//-----------------------------------------------------------------------------
// Include Files
//-----------------------------------------------------------------------------

#include "cheetah_register.h"
#include <vector>


//
// The vector of registered functions.
//
static std::vector<CheetahFunctionEntry*> *function_table_s;
static std::vector<const char*> *function_names_s;
static std::vector<const char*> *function_type_names_s;

//
// Ensure that the static variables are set.
//

static void ensure_static_initializers()
{
  static bool first_time = true;
  if ( first_time )
    {
      function_table_s = new std::vector<CheetahFunctionEntry*>;
      function_names_s = new std::vector<const char*>;
      function_type_names_s = new std::vector<const char*>;
      first_time = false;
    }
}

//
// Register a new function, return the tag for it.
//
void
CheetahFunctionEntry::insert_into_registry(int tag,
					 CheetahFunctionEntry* p,
					 const char *name,
					 const char *type_name)
{
  //
  // Make sure our tables have consistent sizes and that the static
  // initializers have been called.
  //
  ensure_static_initializers();

  assert( function_table_s->size() == function_names_s->size() );
  assert( function_table_s->size() == function_type_names_s->size() );

  //
  // Make sure we have space for this handler.
  //
  if ( tag >= function_table_s->size() )
    {
      function_table_s->resize(tag+1,0);
      function_names_s->resize(tag+1,0);
      function_type_names_s->resize(tag+1,0);
    }

  //
  // Store the function for this tag.
  //
  (*function_table_s)[tag] = p;

  //
  // Save the name of the function.
  //
  char *save_name = 0;
  if ( name )
    {
      save_name = new char[ strlen(name) + 1 ];
      strcpy(save_name,name);
    }
  (*function_names_s)[tag] = save_name;

  //
  // Save the type of the function as a string.
  //
  save_name = 0;
  if ( type_name )
    {
      save_name = new char[ strlen(type_name) + 1 ];
      strcpy(save_name,type_name);
    }
  (*function_type_names_s)[tag] = save_name ;

  //
  // Make sure the tables are still of consistent size.
  //
  assert( function_table_s->size() == function_names_s->size() );
  assert( function_table_s->size() == function_type_names_s->size() );
}

//
// Execute a function with a given tag using the buffer that came
// off the wire.
//
void
CheetahFunctionEntry::run(int tag, char *buffer, int length)
{
  ensure_static_initializers();
  assert( function_table_s->size() == function_names_s->size() );
  assert( function_table_s->size() == function_type_names_s->size() );

  assert(tag>=0);
  assert(tag<function_table_s->size());
  (*function_table_s)[tag]->run(buffer,length);
}

//
// Print out the name of a function in the registry.
//
const char*
CheetahFunctionEntry::name(int tag)
{
  ensure_static_initializers();
  assert( function_table_s->size() == function_names_s->size() );
  assert( function_table_s->size() == function_type_names_s->size() );

  assert(tag>=0);
  assert(tag<function_table_s->size());
  return (*function_names_s)[tag];
}

//
// Print out the type of a function in the registry.
//
const char*
CheetahFunctionEntry::type_name(int tag)
{
  ensure_static_initializers();
  assert( function_table_s->size() == function_names_s->size() );
  assert( function_table_s->size() == function_type_names_s->size() );

  assert(tag>=0);
  assert(tag<function_table_s->size());
  return (*function_type_names_s)[tag];
}

//----------------------------------------------------------------------

void cheetah_dispatch(int who, int tag, void *buffer, int length)
{
  assert( (tag>=0) && (tag<function_table_s->size()) );
  assert( function_table_s->size() == function_names_s->size() );
  assert( function_table_s->size() == function_type_names_s->size() );
  (*function_table_s)[tag]->run((char*)buffer,length);
}


//----------------------------------------------------------------------

#endif   // #if 0
