#ifndef CHEETAH_CORE_REGISTER_H
#define CHEETAH_CORE_REGISTER_H

//-----------------------------------------------------------------------------
// C++ function interface to Cheetah capabilities.
//
// NOTE: The following routines are based on an older design that used
// a 'core' interface and these C++ functions as wrappers around that
// core.  This design has since been changed to use polymorphic 'controllers'
// for the core functionality, and these routines have NOT been updated to
// that new design.  They are here for historical reference.  Because
// of this, they are disabled with an #if 0 / #endif block.
//
//-----------------------------------------------------------------------------

#if 0

//-----------------------------------------------------------------------------
// Include Files
//-----------------------------------------------------------------------------

#include <assert.h>
#include <typeinfo>
#include <utility>
#include "cheetah_register_handler.h"
#include "cheetah_pack.h"


//----------------------------------------------------------------------
// RemoveReference
//
// Given a class T, if it is a reference type, strip off the
// reference to give a regular type.
//----------------------------------------------------------------------

template<class T>
struct RemoveReference
{
  typedef T type_t;
};

template<class T>
struct RemoveReference<T&>
{
  typedef T type_t;
};

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (*)() >
//
// The version of cheetah_execute_handler for functions that take no
// arguments.  Since there are no arguments, don't unpack anything.
//----------------------------------------------------------------------

template<class Ret>
inline void 
cheetah_execute_handler(Ret (*handler)(), char *, int)
{
  assert(handler!=0);
  (*handler)();
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (*)(A1) >
//
// The version of cheetah_execute_handler for functions that take one
// argument.
//----------------------------------------------------------------------

template<class Ret, class A1>
inline void 
cheetah_execute_handler(Ret (*handler)(A1), char *buffer, int length)
{
  assert(buffer!=0);
  assert(length>=0);
  assert(handler!=0);
  //
  // Unpack the one argument from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  int n = cheetah_unpack(p1,buffer,length);
  assert(n<=length);

  //
  // Call the function with this argument.
  //
  (*handler)(*p1);

  //
  // Delete the argument in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (*)(A1,A2) >
//
// The version of cheetah_execute_handler for functions that take two
// arguments.
//----------------------------------------------------------------------

template<class Ret, class A1, class A2>
inline void 
cheetah_execute_handler(Ret (*handler)(A1,A2), char *buffer, int length)
{
  assert(buffer!=0);
  assert(length>=0);
  assert(handler!=0);
  //
  // Unpack the one arguments from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  RemoveReference<A2>::type_t *p2;

  int n = cheetah_unpack(p1,buffer,length);
  assert(n<=length);
  n += cheetah_unpack(p2,buffer+n,length-n);
  assert(n<=length);

  //
  // Call the function with this argument.
  //
  (*handler)(*p1,*p2);

  //
  // Delete the arguments in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
  cheetah_delete_unpacked(p2);
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (*)(A1,A2,A3) >
//
// The version of cheetah_execute_handler for functions that take three
// arguments.
//----------------------------------------------------------------------

template<class Ret, class A1, class A2, class A3>
inline void 
cheetah_execute_handler(Ret (*handler)(A1,A2,A3), char *buffer, int length)
{
  assert(buffer!=0);
  assert(length>=0);
  assert(handler!=0);
  //
  // Unpack the one arguments from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  RemoveReference<A2>::type_t *p2;
  RemoveReference<A3>::type_t *p3;

  int n = cheetah_unpack(p1,buffer,length);
  assert(n<=length);
  n += cheetah_unpack(p2,buffer+n,length-n);
  assert(n<=length);
  n += cheetah_unpack(p3,buffer+n,length-n);
  assert(n<=length);

  //
  // Call the function with this argument.
  //
  (*handler)(*p1,*p2,*p3);

  //
  // Delete the arguments in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
  cheetah_delete_unpacked(p2);
  cheetah_delete_unpacked(p3);
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (*)(A1,A2,A3,A4) >
//
// The version of cheetah_execute_handler for functions that take four
// arguments.
//----------------------------------------------------------------------

template<class Ret, class A1, class A2, class A3, class A4>
inline void 
cheetah_execute_handler(Ret (*handler)(A1,A2,A3,A4), char *buffer, int length)
{
  assert(buffer!=0);
  assert(length>=0);
  assert(handler!=0);
  //
  // Unpack the one arguments from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  RemoveReference<A2>::type_t *p2;
  RemoveReference<A3>::type_t *p3;
  RemoveReference<A4>::type_t *p4;

  int n = cheetah_unpack(p1,buffer,length);
  assert(n<=length);
  n += cheetah_unpack(p2,buffer+n,length-n);
  assert(n<=length);
  n += cheetah_unpack(p3,buffer+n,length-n);
  assert(n<=length);
  n += cheetah_unpack(p4,buffer+n,length-n);
  assert(n<=length);

  //
  // Call the function with this argument.
  //
  (*handler)(*p1,*p2,*p3,*p4);

  //
  // Delete the arguments in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
  cheetah_delete_unpacked(p2);
  cheetah_delete_unpacked(p3);
  cheetah_delete_unpacked(p4);
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (T::*)() >
//
// The version of cheetah_execute_handler for functions that take no
// arguments.  Since there are no arguments, don't unpack anything
// besides the pointer to the object.
//----------------------------------------------------------------------

template<class Ret, class T>
inline void 
cheetah_execute_handler(Ret (T::*handler)(), char * buffer, int length)
{
  assert(handler!=0);
  assert(length >= sizeof(T*));
  assert(buffer!=0);
  T* object = *(T**) buffer;
  (object->*handler)();
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (T::*)(A1) >
//
// The version of cheetah_execute_handler for functions that take one
// argument.
//----------------------------------------------------------------------

template<class Ret, class T, class A1>
inline void 
cheetah_execute_handler(Ret (T::*handler)(A1), char *buffer, int length)
{
  assert(handler!=0);
  assert(buffer!=0);
  int n = sizeof(T*);
  assert( n<=length );

  T* object = *(T**) buffer;

  //
  // Unpack the one argument from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  n += cheetah_unpack(p1,buffer+n,length-n);
  assert( n<=length );

  //
  // Call the function with this argument.
  //
  (object->*handler)(*p1);

  //
  // Delete the argument in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (T::*)(A1,A2) >
//
// The version of cheetah_execute_handler for functions that take two
// arguments.
//----------------------------------------------------------------------

template<class Ret, class T, class A1, class A2>
inline void 
cheetah_execute_handler(Ret (T::*handler)(A1,A2), char *buffer, int length)
{
  int n = sizeof(T*);
  assert(handler!=0);
  assert(buffer!=0);
  assert( n<=length );

  T* object = *(T**) buffer;

  //
  // Unpack the one arguments from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  RemoveReference<A2>::type_t *p2;

  n += cheetah_unpack(p1,buffer+n,length-n);
  assert( n<=length );
  n += cheetah_unpack(p2,buffer+n,length-n);
  assert( n<=length );

  //
  // Call the function with this argument.
  //
  (object->*handler)(*p1,*p2);

  //
  // Delete the arguments in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
  cheetah_delete_unpacked(p2);
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (T::*)(A1,A2,A3) >
//
// The version of cheetah_execute_handler for functions that take three
// arguments.
//----------------------------------------------------------------------

template<class Ret, class T, class A1, class A2, class A3>
inline void 
cheetah_execute_handler(Ret (T::*handler)(A1,A2,A3), char *buffer, int length)
{
  int n = sizeof(T*);
  assert(handler!=0);
  assert(buffer!=0);
  assert( n<=length );

  T* object = *(T**) buffer;

  //
  // Unpack the one arguments from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  RemoveReference<A2>::type_t *p2;
  RemoveReference<A3>::type_t *p3;

  n += cheetah_unpack(p1,buffer+n,length-n);
  assert( n<=length );
  n += cheetah_unpack(p2,buffer+n,length-n);
  assert( n<=length );
  n += cheetah_unpack(p3,buffer+n,length-n);
  assert( n<=length );

  //
  // Call the function with this argument.
  //
  (object->*handler)(*p1,*p2,*p3);

  //
  // Delete the arguments in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
  cheetah_delete_unpacked(p2);
  cheetah_delete_unpacked(p3);
}

//----------------------------------------------------------------------
// cheetah_execute_handler< Ret (T::*)(A1,A2,A3,A4) >
//
// The version of cheetah_execute_handler for functions that take four
// arguments.
//----------------------------------------------------------------------

template<class Ret, class T, class A1, class A2, class A3, class A4>
inline void 
cheetah_execute_handler(Ret (T::*handler)(A1,A2,A3,A4), char *buffer, int length)
{
  int n = sizeof(T*);
  assert(handler!=0);
  assert(buffer!=0);
  assert( n<=length );

  T* object = *(T**) buffer;

  //
  // Unpack the one arguments from the buffer.  If the user
  // function takes its value by reference, strip that off first.
  //
  RemoveReference<A1>::type_t *p1;
  RemoveReference<A2>::type_t *p2;
  RemoveReference<A3>::type_t *p3;
  RemoveReference<A4>::type_t *p4;

  n += cheetah_unpack(p1,buffer,length);
  assert( n<=length );
  n += cheetah_unpack(p2,buffer,length);
  assert( n<=length );
  n += cheetah_unpack(p3,buffer,length);
  assert( n<=length );
  n += cheetah_unpack(p4,buffer,length);
  assert( n<=length );

  //
  // Call the function with this argument.
  //
  (object->*handler)(*p1,*p2,*p3,*p4);

  //
  // Delete the arguments in a manner consistent with how it was
  // unpacked.
  //
  cheetah_delete_unpacked(p1);
  cheetah_delete_unpacked(p2);
  cheetah_delete_unpacked(p3);
  cheetah_delete_unpacked(p4);
}

//----------------------------------------------------------------------
// CheetahFunctionEntry
//
// This is the abstract base class for the classes that will wrap user
// function handlers.  The run function will unpack the arguments from
// the buffer, call the user handler function, and clean up the
// arguments.
//----------------------------------------------------------------------

class CheetahFunctionEntry
{
public:

  //
  // Abstract function to run the user's handler.
  //
  virtual void run(char *buffer, int length) = 0;

  //
  // Abstract function to get a string for the function spec.
  // Useful for debugging.
  //
  virtual const char* name() = 0;

  //
  // Register a new function with a given Cheetah core tag.
  //
  static void insert_into_registry(int tag,
				   CheetahFunctionEntry* p,
				   const char *name,
				   const char *type_name);

  //
  // Execute a function with a given tag using the buffer that came
  // off the wire.
  //
  static void run(int tag, char *buffer, int length);

  //
  // Get a string for the name of a given function.
  //
  static const char* name(int tag);
  static const char* type_name(int tag);
};

//----------------------------------------------------------------------
// CheetahFunctionWrapper<T>
// 
// This is the generic template from which we'll create the
// specializions that handle functions with various numbers of
// arguments.  This is what I call an 'abstract template'.
//----------------------------------------------------------------------

template<class FPtr>
class CheetahFunctionWrapper : public CheetahFunctionEntry
{
  //
  // Store the function pointer.
  //
  FPtr handler_m;

public:

  //
  // Construct with the specific function to call.
  //
  inline CheetahFunctionWrapper(FPtr f) : handler_m(f) {}

  //
  // Execute the function.  Use the function cheetah_execute_handler to
  // actually do the execution.  cheetah_execute_handler is specialized for
  // various types of function pointer, and does the proper unpack and
  // destroy of the arguments.
  //
  void run(char* buffer, int length)
    {
      cheetah_execute_handler(handler_m,buffer,length);
    }

  //
  // Get the name of the function.
  //
  const char *name() { return typeid(handler_m).name(); }
};

//----------------------------------------------------------------------
// Cheetah_register
//
// Register a function with Cheetah.  Return a tag that can be used to
// refer to that function for remote function invocation.
//----------------------------------------------------------------------

void cheetah_dispatch(int who, int tag, void*buffer, int length);

template<class T>
inline int
cheetah_register(T f, char *name=0)
{
  int tag = cheetah_register_handler(cheetah_dispatch);

  CheetahFunctionEntry::insert_into_registry(
                tag,
                new CheetahFunctionWrapper<T>(f) ,
                name,
                typeid(f).name() );
  return tag;
}


//----------------------------------------------------------------------

#endif   // #if 0

#endif // CHEETAH_CORE_REGISTER_H
