#ifndef CHEETAH_CORE_RETRIEVE_H
#define CHEETAH_CORE_RETRIEVE_H

//----------------------------------------------------------------------
// C++ function interface to Cheetah capabilities.
//
// NOTE: The following routines are based on an older design that used
// a 'core' interface and these C++ functions as wrappers around that
// core.  This design has since been changed to use polymorphic 'controllers'
// for the core functionality, and these routines have NOT been updated to
// that new design.  They are here for historical reference.  Because
// of this, they are disabled with an #if 0 / #endif block.
//
//----------------------------------------------------------------------

#if 0

//----------------------------------------------------------------------
// Include Files
//----------------------------------------------------------------------

#include <assert.h>
#include "cheetah_core.h"
#include "cheetah_register.h"
#include "cheetah_ainvoke.h"


//----------------------------------------------------------------------

template<class T>
struct CheetahRetrieve
{
  static T* retrieve(int context, T* object);
  static void handle_request(int who, T* object, T** requester, int *bell);
  static void handle_reply(T& object, T** requester, int *bell);
  static const int request_tag_s;
  static const int reply_tag_s;
};

template<class T>
const int CheetahRetrieve<T>::request_tag_s = 
  cheetah_register(CheetahRetrieve<T>::handle_request,
		 "handle_retrieve_request");

template<class T>
const int CheetahRetrieve<T>::reply_tag_s = 
  cheetah_register(CheetahRetrieve<T>::handle_reply,
		 "handle_retrieve_reply");


//----------------------------------------------------------------------
//
// CheetahRetrieve::retrieve
//
// Get the object from a remote context and store it in our heap.  It
// is the responsibility of the caller to free the storage.
//
//----------------------------------------------------------------------

template<class T>
T*
CheetahRetrieve<T>::retrieve(int context, T* object) 
{
  assert( (context>=0) && (context<cheetah_core_ncontexts()) );
  assert( object!=0 );

  //
  // A pointer for the object when it arrives and the bell to ring
  // when it gets here.
  //
  T *p=0;
  int bell=0;

  //
  // Tell the other context to send it here.  The reply will unpack
  // the object and put the pointer to it in p.
  //
  cheetah_ainvoke(context,request_tag_s,0,
		cheetah_core_mycontext(), object, &p, &bell);

  //
  // When it arrives it will ring the bell.
  //
  cheetah_core_wait(&bell,1);

  //
  // Make sure we got something, and return it.
  //
  assert( p!=0 );
  return p;
}

//----------------------------------------------------------------------

template<class T>
void CheetahRetrieve<T>::
handle_request(int who, T* object,T** requester, int* bell)
{
  assert((who>=0)&&(who<cheetah_core_ncontexts()));
  assert(object!=0);
  assert(requester!=0);
  assert(bell!=0);
  cheetah_ainvoke(who,reply_tag_s,0,*object,requester,bell);
}

//----------------------------------------------------------------------

template<class T>
void CheetahRetrieve<T>::
handle_reply(T& object, T** requester, int* bell)
{
  assert(requester!=0);
  assert(bell!=0);
  *requester = new T(object);
  ++*bell;
}

//----------------------------------------------------------------------

#endif   // #if 0

#endif // CHEETAH_CORE_RETRIEVE_H
