#ifndef CHEETAH_CORE_STORE_H
#define CHEETAH_CORE_STORE_H

//----------------------------------------------------------------------
// C++ function interface to Cheetah capabilities.
//
// NOTE: The following routines are based on an older design that used
// a 'core' interface and these C++ functions as wrappers around that
// core.  This design has since been changed to use polymorphic 'controllers'
// for the core functionality, and these routines have NOT been updated to
// that new design.  They are here for historical reference.  Because
// of this, they are disabled with an #if 0 / #endif block.
//
//----------------------------------------------------------------------

#if 0

//----------------------------------------------------------------------
// Include Files
//----------------------------------------------------------------------

#include <stdio.h>
#include <assert.h>
#include "cheetah_core.h"
#include "cheetah_register.h"
#include "cheetah_ainvoke.h"


//----------------------------------------------------------------------

template<class T>
struct CheetahStore
{
  static void store(int context, T* remote, const T& local, int op);
  static void handle_request(T& object, T* destination, int op);
  static const int request_tag_s;
  enum { ASSIGN=1, ADD_ASSIGN, SUB_ASSIGN, MUL_ASSIGN, DIV_ASSIGN };
};

template<class T>
const int CheetahStore<T>::request_tag_s = 
  cheetah_register(CheetahStore<T>::handle_request,
		 "handle_store_request");


//----------------------------------------------------------------------
//
// CheetahStore::store
//
// Take a reference to a local object and store it to a remote context
// at a given location.  Us the full object pack/unpack techniques.
//
//----------------------------------------------------------------------

template<class T>
void
CheetahStore<T>::store(int context, T* remote, const T& local, int op) 
{
  assert( (context>=0) && (context<cheetah_core_ncontexts()) );
  assert( remote!=0 );

  //
  // A bell to ring when the object is on its way.
  //
  int bell=0;

  //
  // Send the object to the other context.
  //
  cheetah_ainvoke(context,request_tag_s,&bell, local, remote, op);

  //
  // When it is on its way it will ring the bell.
  //
  cheetah_core_wait(&bell,1);
}

//----------------------------------------------------------------------

template<class T>
void CheetahStore<T>::
handle_request(T& object, T* destination, int op)
{
  assert(destination!=0);
  switch (op)
    {
    case ASSIGN:
      *destination = object;
      break;
    case ADD_ASSIGN:
      *destination += object;
      break;
    case SUB_ASSIGN:
      *destination -= object;
      break;
    case MUL_ASSIGN:
      *destination *= object;
      break;
    case DIV_ASSIGN:
      *destination /= object;
      break;
    default:
      fprintf(stderr,"%d: Invalid store operation request %d!\n",
	      cheetah_core_mycontext(),op);
      break;
    }
}

//----------------------------------------------------------------------

#endif   // #if 0

#endif // CHEETAH_CORE_STORE_H
