#ifndef CHEETAH_UTILITIES_SPINLOCKMUTEX_H
#define CHEETAH_UTILITIES_SPINLOCKMUTEX_H

//-----------------------------------------------------------------------------
// Classes: 
//   SpinlockMutex
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Overview:
//   SpinlockMutex - a mutex implementation using spinlock mechanisms.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include Files:
//-----------------------------------------------------------------------------

namespace Cheetah {

//-----------------------------------------------------------------------------
//
// class SpinlockMutex
//
// Implements a spinlock mutex.
//
// This class allows mutual exclusive access to a critical section by
// spinning on a lock. The implementation uses low level assembly
// instructions to provide mutual exclusion.
//
//-----------------------------------------------------------------------------

class SpinlockMutex
{
public:

  SpinlockMutex();

  //
  // Acquire the lock. Calling thread waits until the lock is granted.
  //
  void lock();

  //
  // Try to acquire the lock. Returns 1 if the lock was granted, zero
  // otherwise.
  //
  int trylock();

  //
  // Releases the lock.
  //
  void unlock();

private:
  volatile unsigned int slock;
};


} // namespace Cheetah


//-----------------------------------------------------------------------------
// The C interface to spinlock mutex mechanism.
// The implementation of these routines are provided in
// architecture-specific assembly routines.
//-----------------------------------------------------------------------------

extern "C"
{
  void spinlock(volatile unsigned int *, volatile unsigned int);
  int spintrylock(volatile unsigned int *, volatile unsigned int);
}


#endif // CHEETAH_UTILITIES_SPINLOCKMUTEX_H
